/*
 * Copyright 2017 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.guvnor.ala.ui.backend.service;

import java.util.Arrays;

import org.guvnor.ala.ui.service.ProvisioningValidationService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.junit.MockitoJUnitRunner;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

@RunWith(MockitoJUnitRunner.class)
public class ProvisioningValidationServiceImplTest {

    private ProvisioningValidationService service;

    private static final String[] validContainerNames = {"valid",
            "valid1",
            "valid-1",
            "Valid1",
            "VALID1",
            "valid_1",
            "valid:1"};

    private static final String[] invalidContainerNames = {null,
            "",
            " ",
            "á",
            "@",
            "inva lid",
            " invalid",
            "invalid ",
            "inva,lid"};

    @Before
    public void setUp() {
        service = new ProvisioningValidationServiceImpl();
    }

    @Test
    public void testIsValidContainerNameForValidNames() {
        Arrays.stream(validContainerNames).forEach(name -> assertTrue(service.isValidContainerName(name)));
    }

    @Test
    public void testIsValidContainerNameForInvalidNames() {
        Arrays.stream(invalidContainerNames).forEach(name -> assertFalse(service.isValidContainerName(name)));
    }
}
